
local cinematic = {}

local cinematics = {}

local function savecallbacks(callbacktable)
	callbacktable.draw = love.draw
	callbacktable.update = love.update
	callbacktable.keypressed = love.keypressed
	callbacktable.gamepadpressed = love.gamepadpressed
	callbacktable.touchmoved = love.touchmoved
	callbacktable.handleInput = handleInput
end

local function restorecallbacks(callbacktable)
	love.draw = callbacktable.draw
	love.update = callbacktable.update
	love.keypressed = callbacktable.keypressed
	love.gamepadpressed = callbacktable.gamepadpressed
	love.touchmoved = callbacktable.touchmoved
	handleInput = callbacktable.handleInput
end

function cinematic.play(cinematicname)
	local oldfunctions = {}
	savecallbacks(oldfunctions)

	local c = cinematics[cinematicname]

	love.draw = function()
		c:draw()
	end

	love.update = function(dt)
		c:update(dt)

		if c:done() then
			c:unload()
			restorecallbacks(oldfunctions)
		end
	end

	love.keypressed = function()
		c.state = -1
	end

	love.gamepadpressed = function (joystick, button)
		if button == "start" then
			c.state = -1
		end
	end

	love.touchmoved = nil
	love.handleInput = function() end

	c:load()
end

-------------------------------------------------
-- Opening
-------------------------------------------------

cinematics.opening = {}

function cinematics.opening:load()
	--self.variant = not self.variant
	self.variant = false

	self.time = 0
	self.state = 1

	self.entities = {}

	self.background = love.graphics.newImage("img/intro_background.png")
	self.water = love.graphics.newImage("img/intro_water.png")
	self.water:setWrap("repeat")
	local iw, ih = self.water:getDimensions()
	self.waterquad = love.graphics.newQuad(0, 0, iw, 540, iw, ih)

	self.submarine = {
		x = self.variant and -30 or -90, y = 150,
		image = love.graphics.newImage("img/submarine_submerged_dark.png"),
	}
	self.cannonfire = {
		x = -100, y = -100,
		image = love.graphics.newImage("img/cannonball_fire_dark.png"),
	}
	self.cannonfire.animation = hajtech.animation.new(self.cannonfire.image, 8, 0, 6, 64)
	self.cannonfire.animation.wrap = false
	self.cannonfire.animation.speed = 0
	self.cannonball = {
		x = -100, y = -100,
		image = love.graphics.newImage("img/cannonball_trail.png"),
	}
	self.cannonball.animation = hajtech.animation.new(self.cannonball.image, 8, 0, 3, 96, 64)
	self.enemy = {
		x = self.variant and 1200 or 1500, y = 0,
		image = love.graphics.newImage("img/enemy_ship.png"),
	}

	table.insert(self.entities, self.submarine)
	table.insert(self.entities, self.cannonfire)
	table.insert(self.entities, self.cannonball)
	table.insert(self.entities, self.enemy)

	self:update(0)
end

function cinematics.opening:unload()
	self.time = nil
	self.background = nil
	self.submarine = nil
	self.cannonfire = nil
	self.cannonball = nil
	self.enemy = nil
end

function cinematics.opening:draw()
	hajtech.rendering.drawbegin()

	love.graphics.clear({10, 30, 100})
	if self.variant then
		love.graphics.draw(self.background, 0, 0)
	else
		love.graphics.draw(self.water, self.waterquad, -self.time*1500, 0, 0, 5, 5)
	end

	for _, entity in ipairs(self.entities) do
		if entity.animation then
			love.graphics.draw(
				entity.image,
				entity.animation:getFrame(),
				math.floor(entity.x),
				math.floor(entity.y))
		else
			love.graphics.draw(
				entity.image,
				math.floor(entity.x),
				math.floor(entity.y))
		end
	end

	hajtech.rendering.drawend()
end

function cinematics.opening:update(dt)
	self.time = self.time + dt

	self.cannonfire.animation:update(dt)
	self.cannonball.animation:update(dt)

	self.submarine.y = self.submarine.y + 5*math.sin(self.time)*dt

	if self.state == 1 then
		if self.submarine.x < (self.variant and 0 or -40) then
			self.submarine.x = math.min(0, self.submarine.x + 16*dt)
			self.enemy.x = self.enemy.x - (self.variant and 180 or 0)*dt
		else
			self.state = self.state + 1
		end
	elseif self.state == 2 then
		if not self.variant then self.submarine.x = math.min(0, self.submarine.x + 16*dt) end
		if self.time >= 4 then
			self.state = self.state + 1
			self.cannonfire.animation.speed = 1
		end
	elseif self.state == 3 then
		if not self.variant then self.submarine.x = math.min(0, self.submarine.x + 16*dt) end
		if self.cannonfire.animation._currentFrame == self.cannonfire.animation._frameEnd then
			self.state = self.state + 1
			self.cannonball.x = self.cannonfire.x + 32
			self.cannonball.y = self.cannonfire.y
		end
	elseif self.state == 4 then
		if not self.variant then self.submarine.x = math.min(0, self.submarine.x + 16*dt) end
		if self.cannonball.x < 800 then
			self.cannonball.x = self.cannonball.x + 550*dt
			self.enemy.x = self.enemy.x - (self.variant and 0 or 700)*dt
		else
			self.state = self.state + 1
		end
	elseif self.state == 5 then
		if self.time >= 1 then
			self.state = -1
		end
	end

	self.cannonfire.x = self.submarine.x + 256
	self.cannonfire.y = self.submarine.y + 128
end

function cinematics.opening:done()
	return self.state < 0
end

-------------------------------------------------

return cinematic
