
local block = {}

local sprite = require('entities/sprite')

function block.create(object)
	object.properties.solid = true
	object.properties.pushable = true
	object.update = block.update
	object.draw = block.draw
	object.push = block.push

	object.maxspeed = object.properties.maxspeed or 10
	object.yspeed = 0
	object.onground = false

	object.rotation = 0
	object.animation = false
	object.framedistance = object.properties.framedistance
	object.skin = object.properties.skin or 0

	if object.properties.image then
		object.image = love.graphics.newImage(object.properties.image)
		object.image:setFilter("nearest", "nearest")
	end
	if object.properties.rotateimage then
		object.rotateimage = love.graphics.newImage(object.properties.rotateimage)
		object.rotateimage:setFilter("nearest", "nearest")
		object.w = object.w - 6
	end
	if object.properties.impactimage then
		object.impactimage = love.graphics.newImage(object.properties.impactimage)
		object.impactimage:setFilter("nearest", "nearest")
		local iw, ih = object.impactimage:getDimensions()
		local animation = hajtech.animation.new(object.impactimage,
			5,
			0, iw/ih - 1,
			ih, ih)
		object.impactsprite = sprite.new(0, 0, object.impactimage, animation, -1)
	end
end

function block:update(dt)
	if not self.animation then
		self.animation = true	-- only try once

		local tile = self.gid and hajtech.map.tiles[self.gid]
		if tile and tile.tileset then
			local iw = tile.tileset.imagewidth
			local ih = tile.tileset.imageheight
			local tw = tile.tileset.tilewidth
			local th = tile.tileset.tileheight

			self.image = tile.tileset.image
			self.tile = tile
			if self.framedistance then
				self.animation = hajtech.animation.new(self.image, 1, 0, iw/tw-1, tw, th, 0, self.skin*th)
			else
				self.quad = tile.quad
			end
		elseif self.image then
			local iw, ih = self.image:getDimensions()
			local tw, th = ih, ih
			self.animation = hajtech.animation.new(self.image, 1, 0, iw/tw-1, tw, th)
		end
	end

	if self.impactsprite then
		self.impactsprite:update(dt)
	end

	if hajtech.collision.moveY(self, self.yspeed*dt) then
		if self.yspeed > 0 then
			self.onground = true

			if self.yspeed >= 128 then
				if self.impactimage then
					self.impactsprite:reset(self.x + self.w/2, self.y + self.h/2)

					local search = {
						x = self.x - 32*3,
						y = self.y,
						w = self.w + 32*6,
						h = self.h
					}
					local matches = hajtech.collision.overlapWithEntity(search, 'character')
					for _, player in ipairs(matches) do
						if player.onground then
							local dx = self.x - player.x

							player.xspeed = -sign(dx) * 100
							player.yspeed = -100
							player.onground = false
						end
					end
				end
			end
		end

		self.yspeed = 0
	else
		self.onground = false
	end

	-- Apply gravity
	self.yspeed = math.min(256, self.yspeed + 600*dt)
end

function block:draw()
	if self.image then
		local iw, ih = self.image:getDimensions()
		if self.tile then
			iw = self.tile.tileset.tilewidth
			ih = self.tile.tileset.tileheight
		end
		love.graphics.draw(
			self.image,
			self.quad or self.animation:getFrame(),
			self.x+self.w/2,
			self.y+self.h/2,
			0,
			nil,
			nil,
			iw/2,
			ih/2)
	end
	if self.rotateimage then
		local iw, ih = self.rotateimage:getDimensions()
		love.graphics.draw(
			self.rotateimage,
			self.x+self.w/2,
			self.y+self.h/2,
			self.rotation,
			nil,
			nil,
			iw/2,
			ih/2)
	end

	if self.impactsprite and not self.impactsprite.animation:done() then
		self.impactsprite:draw()
	end
end

function block:push(dx)
	local oldx = self.x
	hajtech.collision.moveX(self, dx)
	local movement = self.x - oldx

	self.rotation = self.rotation + movement / (self.w/2)
	if self.framedistance then
		self.animation:update(movement / self.framedistance)
	end

	return movement
end

return block
