
local pirate = {}

local rope = require('entities/characters/rope')

local texture = love.graphics.newImage('img/character-pirate.png')
local viewport_border = love.graphics.newImage('img/player1_border.png')

function pirate.create(object)
	object.properties.character = "pirate"
	object.title = "Captain Deadeye"

	object.reset = pirate.reset
	object.press_jump = pirate.press_jump
	object.update = pirate.update
	object.press_action1 = pirate.launchrope
	object.release_action1 = pirate.droprope
	object.draw = pirate.draw
	object.rope = rope.new(object)
	object.rope.texture = texture
	object.rope.quad = love.graphics.newQuad(220, 334, 10, 10, texture:getDimensions())
	object.viewport_border = viewport_border

	object.facing = 1

	-- Animations
	object.texture = texture
	local framesize = 64

	object.animations = {
		idle = hajtech.animation.new(texture, 2, 0, 1, framesize),
		walking = hajtech.animation.new(texture, 10, 7, 12, framesize),
		dieing = hajtech.animation.new(texture, 8, 14, 19, framesize),
		climbing = hajtech.animation.new(texture, 8, 28, 29, framesize),
		hooking = hajtech.animation.new(texture, 20, 35, 36, framesize),
		attacking = hajtech.animation.new(texture, 3, 42, 47, framesize),
		hanging = hajtech.animation.new(texture, 3, 49, 49, framesize),
		jumping = hajtech.animation.new(texture, 0, 7, 7, framesize),
		falling = hajtech.animation.new(texture, 20, 21, 21, framesize),
		--falling = hajtech.animation.new(texture, 20, 21, 24, framesize),
	}
	object.animations.hooking.wrap = false

	object:reset()
end

function pirate:reset()
	self.w = 22
	self.h = 56
	self.xspeed = 0
	self.yspeed = 0
	self.onground = false
	self.onladder = false
	self.rope.state = false
	self.currentanimation = self.animations.idle
end

function pirate:launchrope()
	if not self.onladder then
		self.rope:launchrope()
	end
end

function pirate:droprope()
	self.rope:droprope()
end

function pirate:press_jump()
	if self.onground or self.onladder then
		self.yspeed = -120
		self.onladder = false

		if hajtech.input.getButtons(self.player).down then
			self.yspeed = 0
			hajtech.collision.moveY(self, 4)
		end
	end
end

function pirate:draw()
	local x, y = self.x + self.w/2, self.y + self.h
	local sx = 1
	local rot = 0

	if self.facing < 0 and not self.onladder then
		sx = -sx
	end

	if self.viewport == nil then
		love.graphics.setShader(hajtech.grayscale)
	end
	local w, h = self.currentanimation:getDimensions()
	if self.rope.state == 'hang' then
		y = self.y + self.h/2
		rot = self.rope.angle - math.rad(90)
		love.graphics.draw(texture, self.currentanimation:getFrame(),
			math.floor(x), math.floor(y), rot, sx, 1, w/2, h/2)
		return
	end

	love.graphics.draw(texture, self.currentanimation:getFrame(),
		math.floor(x), math.floor(y), rot, sx, 1, w/2, h)
	love.graphics.setShader(nil)
end

function pirate:update(dt)
	self.currentanimation:update(dt)

	local x, y = hajtech.input.getXY(self.player)
	local buttons = hajtech.input.getButtons(self.player)
	local nextanimation = "idle"

	if self.onladder then
		self.yspeed = 0

		local ground = hajtech.collision.moveY(self, y*96*dt)
		if y > 0 and ground and ground.properties.solid then
			self.onladder = false
			self.justleftladder = true
		end

		local ladder, lx, ly = hajtech.collision.overlapWith(self, 'ladder')
		if ladder then
			nextanimation = "climbing"

			local diff = lx - (self.x + self.w/2)
			if x ~= 0 and sign(diff) ~= sign(x) and math.abs(y) < 0.2 and math.abs(diff) > 5 then
				self.onladder = false
			else
				hajtech.collision.moveX(self, clamp(diff, 60*dt))
			end
		else
			self.onladder = false
		end
	elseif y ~= 0 and not self.rope.state and not self.justleftladder then
		self.h = self.h + ((y > 0) and 1 or 0)
		self.w = self.w / 2
		self.x = self.x + self.w/2
		local ladder, lx, ly = hajtech.collision.overlapWith(self, 'ladder')
		self.x = self.x - self.w/2
		self.w = self.w * 2
		local solid = hajtech.collision.overlapWith(self, 'solid')
		self.h = self.h - ((y > 0) and 1 or 0)
		if ladder then
			if not solid then
				nextanimation = "climbing"

				self.onladder = true
				self.yspeed = 0
			else
				local ladder = hajtech.collision.overlapWith(self, 'ladder')
				if not ladder then
					local diff = lx - (self.x + self.w/2)
					x = sign(diff)
				end
			end
		end
	end
	if y == 0 then
		self.justleftladder = false
	end
	if self.onladder then
		self.animations.climbing.speed = y
	end

	if x > 0 then
		self.facing = 1
	elseif x < 0 then
		self.facing = -1
	end

	if self.onground then
		if self.xspeed ~= 0 then
			nextanimation = "walking"
			self.animations.walking.speed = math.abs(x)
		end
	elseif not self.onladder then
		if self.yspeed > 0 then
			nextanimation = "falling"
		else
			nextanimation = "jumping"
		end
	end

	if self.rope.state then
		if self.rope.state == 'hang' then
			nextanimation = "hanging"
		else
			if self.onground then
				nextanimation = "hooking"
			else
				nextanimation = "hooking" -- hooking in air
			end
		end
	end

	if hajtech.collision.moveX(self, self.xspeed*dt) then
		if not self.onladder then
			self.xspeed = 0
		end
	end

	if hajtech.collision.moveY(self, self.yspeed*dt) then
		if self.yspeed > 0 then
			self.onground = true
		end

		self.yspeed = 0
	else
		self.onground = false
	end

	-- xspeed
	local targetspeed = x*160
	local dspeed = targetspeed - self.xspeed
	if sign(targetspeed) ~= sign(self.xspeed) and targetspeed ~= 0 then
		-- retardaccel
		speedchange = 1200*dt
		dspeed = clamp(dspeed, speedchange)
	elseif math.abs(targetspeed) < math.abs(self.xspeed) then
		-- retard
		speedchange = 800*dt
		dspeed = clamp(dspeed, speedchange)
	else
		-- accel
		speedchange = 400*dt
		dspeed = clamp(dspeed, speedchange)
	end
	self.xspeed = self.xspeed + dspeed

	-- Apply gravity
	self.yspeed = math.min(256, self.yspeed + 600*dt)

	-- Disregard gravity
	if buttons.fly then
		self.yspeed = 128*y
	end

	self:transit(nextanimation)
end

return pirate
