
local sailor = {}

local texture = love.graphics.newImage('img/character-sailor.png')
local viewport_border = love.graphics.newImage('img/player2_border.png')

function sailor.create(object)
	object.properties.character = "sailor"
	object.title = "Boatswain Billy"

	object.reset = sailor.reset
	object.draw = sailor.draw
	object.update = sailor.update
	object.press_jump = sailor.press_jump
	object.press_use = sailor.press_use
	object.press_action1 = sailor.pickup
	object.release_action1 = sailor.throw
	object.retake_viewport = sailor.retake_viewport
	object.viewport_border = viewport_border

	object.facing = 1

	-- Animations
	object.texture = texture
	local framesize = 64

	object.animations = {
		idle = hajtech.animation.new(texture, 2, 0, 1, framesize),
		walking = hajtech.animation.new(texture, 10, 8, 15, framesize),
		dieing = hajtech.animation.new(texture, 8, 16, 19, framesize),
		climbing = hajtech.animation.new(texture, 8, 32, 33, framesize),
		climbingrope = hajtech.animation.new(texture, 8, 34, 35, framesize),
		falling = hajtech.animation.new(texture, 0, 24, 24, framesize),
		jumping = hajtech.animation.new(texture, 0, 14, 14, framesize),
		throwing = hajtech.animation.new(texture, 1, 41, 41, framesize),
		throwing2 = hajtech.animation.new(texture, 1, 44, 44, framesize),
		pushing = hajtech.animation.new(texture, 10, 48, 55, framesize),
	}

	object:reset()
end

function sailor:reset()
	self.w = 24
	self.h = 56
	self.xspeed = 0
	self.yspeed = 0
	self.onground = false
	self.onladder = false
	self.throwing = false
	self.carrying = nil
	self.currentanimation = self.animations.idle
end

function sailor:draw()
	local x, y = self.x + self.w/2, self.y + self.h
	local sx = 1
	local rot = 0

	if self.facing < 0 and not self.onladder then
		sx = -sx
	end

	if self.viewport == nil then
		love.graphics.setShader(hajtech.grayscale)
	end
	local w, h = self.currentanimation:getDimensions()
	love.graphics.draw(texture, self.currentanimation:getFrame(),
		math.floor(x), math.floor(y), rot, sx, 1, w/2, h)

	if self.carrying and self.carrying.draw then
		self.carrying:draw()
	end

	if self.throwing then
		local tx, ty = self.x + self.w/2, self.y + self.h/2
		ty = ty - math.sin(self.throwangle) * 50
		tx = tx + math.cos(self.throwangle) * 50 * self.facing
		love.graphics.circle('fill', tx, ty, 5, 10)
	end
	love.graphics.setShader(nil)
end

function sailor:update(dt)
	self.currentanimation:update(dt)

	local x, y = hajtech.input.getXY(self.player)
	local buttons = hajtech.input.getButtons(self.player)
	local nextanimation = "idle"

	if self.onladder then
		self.yspeed = 0

		local ground = hajtech.collision.moveY(self, y*96*dt)
		if y > 0 and ground and ground.properties.solid then
			self.onladder = false
			self.justleftladder = true
		end

		local ladder, lx, ly = hajtech.collision.overlapWith(self, 'ladder')
		if not ladder then
			ladder, lx, ly = hajtech.collision.overlapWith(self, 'rope')
		end
		if ladder then
			nextanimation = "climbing"
			if ladder.properties['rope'] then
				nextanimation = "climbingrope"
			end

			local diff = lx - (self.x + self.w/2)
			if x ~= 0 and sign(diff) ~= sign(x) and math.abs(y) < 0.2 and math.abs(diff) > 5 then
				self.onladder = false
			else
				hajtech.collision.moveX(self, clamp(diff, 60*dt))
			end
		else
			self.onladder = false
		end
	elseif y ~= 0 and not self.justleftladder then
		self.h = self.h + ((y > 0) and 1 or 0)
		self.w = self.w / 2
		self.x = self.x + self.w/2
		local ladder, lx, ly = hajtech.collision.overlapWith(self, 'ladder')
		if not ladder then
			ladder, lx, ly = hajtech.collision.overlapWith(self, 'rope')
		end
		self.x = self.x - self.w/2
		self.w = self.w * 2
		local solid = hajtech.collision.overlapWith(self, 'solid')
		self.h = self.h - ((y > 0) and 1 or 0)
		if ladder then
			if not solid then
				nextanimation = "climbing"
				if ladder.properties['rope'] then
					nextanimation = "climbingrope"
				end

				self.onladder = true
				self.yspeed = 0
			else
				local ladder = hajtech.collision.overlapWith(self, 'ladder')
				if not ladder then
					ladder = hajtech.collision.overlapWith(self, 'rope')
				end
				if not ladder then
					local diff = lx - (self.x + self.w/2)
					x = sign(diff)
				end
			end
		end
	end
	if y == 0 then
		self.justleftladder = false
	end
	if self.onladder then
		self.animations.climbing.speed = y
		self.animations.climbingrope.speed = y
	end

	if x > 0 then
		self.facing = 1
	elseif x < 0 then
		self.facing = -1
	end

	if self.onground then
		if self.xspeed ~= 0 then
			nextanimation = "walking"
			self.animations.walking.speed = math.abs(x)
		end
	elseif not self.onladder then
		if self.yspeed > 30 then
			nextanimation = "falling"
		else
			nextanimation = "jumping"
		end
	end

	if self.throwing then
		self.xspeed = 0

		self.throwanglecycle = self.throwanglecycle + 2*dt
		self.throwangle = math.rad(30) + math.sin(self.throwanglecycle)*math.rad(45)
		nextanimation = "throwing"
	end

	local wall = hajtech.collision.moveX(self, self.xspeed*dt)
	if wall then
		if wall.properties.pushable and not self.onladder then
			local pushspeed = clamp(self.xspeed, wall.maxspeed)
			local pushlength = pushspeed*dt
			pushlength = wall:push(pushlength)
			hajtech.collision.moveX(self, pushlength)
			nextanimation = "pushing"
			self.animations.pushing.speed = math.max(math.abs(pushspeed/160), 0.2)
		else
			if not self.onladder then
				self.xspeed = 0
			end
		end
	end

	local ground = hajtech.collision.moveY(self, self.yspeed*dt)
	if ground then
		if self.yspeed > 0 then
			self.onground = true
		end

		self.yspeed = 0

		if ground.properties.spikes then
			self.xspeed = -ground.sx * 300
			self.yspeed = -100
			self.onground = false
		end
	else
		self.onground = false
	end

	if self.carrying then
		local object = self.carrying
		object.facing = self.facing

		local offsetcarryingX
		local offsetcarryingY

		if object.properties.character  == "bunny" then
			if nextanimation == "throwing" then
				-- kaninen ska vara 23 pixlar till vänster om handen
				offsetcarryingX = -28
				offsetcarryingY = 1
			else
				offsetcarryingX = -12
				offsetcarryingY = -14
			end
		else -- other stuff
			if nextanimation == "throwing" then
				-- kaninen ska vara 23 pixlar till vänster om handen
				offsetcarryingX = -28
				offsetcarryingY = 5
			else
				offsetcarryingX = -9
				offsetcarryingY = 11
			end
		end

		object.x = self.x + self.w/2 - object.w/2 + offsetcarryingX * self.facing
		object.y = self.y + self.h/2 - object.h/2 + offsetcarryingY
	end

	-- xspeed
	local targetspeed = x*160
	local dspeed = targetspeed - self.xspeed
	if sign(targetspeed) ~= sign(self.xspeed) and targetspeed ~= 0 then
		-- retardaccel
		speedchange = 1200*dt
		dspeed = clamp(dspeed, speedchange)
	elseif math.abs(targetspeed) < math.abs(self.xspeed) then
		-- retard
		speedchange = 800*dt
		dspeed = clamp(dspeed, speedchange)
	else
		-- accel
		speedchange = 400*dt
		dspeed = clamp(dspeed, speedchange)
	end
	self.xspeed = self.xspeed + dspeed

	-- Apply gravity
	self.yspeed = math.min(256, self.yspeed + 600*dt)

	-- Disregard gravity
	if buttons.fly then
		self.yspeed = 128*y
	end

	self:transit(nextanimation)
end

function sailor:retake_viewport()
	self.viewport.entity = nil
end

function sailor:press_jump()
	self:retake_viewport()
	if self.onground or self.onladder then
		self.yspeed = -120
		self.onladder = false

		if hajtech.input.getButtons(self.player).down then
			self.yspeed = 0
			hajtech.collision.moveY(self, 4)
		end
	end
end

function sailor:press_use()
	self:retake_viewport()
	local matches = hajtech.collision.overlapWithEntity(self, "usable")
	for _, entity in ipairs(matches) do
		if entity.use then
			entity:use(self)
		end
	end
end

function sailor:pickup()
	self:retake_viewport()
	if not self.carrying then
		local object = hajtech.collision.overlapWithEntity(self, "throwable")[1]
		if object then
			self.carrying = object
			object.carriedby = self
			hajtech.map:removeEntity(object)
			if object.transit then
				object:transit("idle")
			end
		else
			local supply = hajtech.collision.overlapWithEntity(self, "supply")[1]
			if supply then
				supply:use(self)
			end
		end
	elseif not self.onladder then
		self.throwing = true
		self.throwangle = math.rad(-30)
		self.throwanglecycle = 0
	end
end

function sailor:throw()
	if self.carrying and self.throwing then
		local object = self.carrying
		object.x, object.y = self.x + self.w/2, self.y + self.h/2
		hajtech.map:addEntity(object)

		object.carriedby = nil
		self.carrying = nil

		object.xspeed = 700 * math.cos(self.throwangle) * self.facing
		object.yspeed = -700 * math.sin(self.throwangle)

		if object.type ~= "character" then
			self.viewport.entity = object
			object.viewport = self.viewport
		end
	end
	self.throwing = false
end

return sailor
