
local platform = {}

function platform.create(object)
	object.setstate = platform.setstate
	object.update = platform.update
	object.draw = platform.draw
	object.state = false

	object.originX, object.originY = object.x, object.y
	object.targetX, object.targetY = object.x, object.y

	object.properties.solid = true
	object.properties.lefttile = object.properties.lefttile or -1
	object.properties.righttile = object.properties.righttile or 1
end

function platform:setstate(state)
	self.state = state
	if state then
		self.targetX = self.originX + (self.properties.slideX or 0)
		self.targetY = self.originY + (self.properties.slideY or 0)
	else
		self.targetX = self.originX
		self.targetY = self.originY
	end

	-- sync other buttons controlling this entity
	for _, source in ipairs(self.incoming_links or {}) do
		if source.type == "button" and source.setstate then
			source:setstate(state)
		end
	end
end

function platform:update(dt)
	-- Compute platform movement
	local dx = self.targetX - self.x
	dx = clamp(dx, (self.properties.speed or 100)*dt)
	local dy = self.targetY - self.y
	dy = clamp(dy, (self.properties.speed or 100)*dt)

	-- If moving into a player on the way down, stop vertical movement
	if self.properties.stopable and dy > 0 then
		self.y = self.y + dy
		self.x = self.x + dx
		local next = next
		if next(hajtech.collision.overlapWithEntity(self, 'character')) ~= nil then
			self.y = self.y - dy
			dy = 0
		end
		self.y = self.y - dy
		self.x = self.x - dx
	end

	-- If no movement then there is nothing to do but update loop_timer
	if dx == 0 and dy == 0 then
		if self.properties.loop then
			if not self.looptimer then
				self.looptimer = tonumber(self.properties.loop) or 0
			elseif self.looptimer > 0 then
				self.looptimer = self.looptimer - dt
			else
				self.looptimer = nil
				self:setstate(not self.state)
			end
		end
		return
	end

	-- Grab things standing on the platform
	self.y = self.y - 2
	local matches = hajtech.collision.overlapWithEntity(self)
	self.y = self.y + 2

	-- Move the platform before the character if going down
	if dy > 0 then
		self.y = self.y + dy
	end

	-- Move along things that were standing on top
	for _, entity in pairs(matches) do
		if entity.player or entity.properties.type == "supply" or entity.properties.type == "block" then
			if self.properties.slideX then
				hajtech.collision.moveX(entity, dx)
			end
			if self.properties.slideY then
				hajtech.collision.moveY(entity, dy)
			end
		end
	end

	-- Move the platform after the character if going up
	if dy < 0 then
		self.y = self.y + dy
	end
	self.x = self.x + dx

	-- Move followers:
	for _, follower in ipairs(self.followers or {}) do
		follower.x = follower.x + dx
		follower.y = follower.y + dy
	end

	-- Push a player you move into
	local matches = hajtech.collision.overlapWithEntity(self)
	for _, entity in pairs(matches) do
		if entity.player then
			if dx < 0 then
				entity.x = self.x - entity.w
			elseif dx > 0 then
				entity.x = self.x + self.w
			end
		end
	end
end

function platform:draw()
	local tw = hajtech.map.tilewidth
	local th = hajtech.map.tileheight
	local left_tile = hajtech.map.tiles[self.gid + self.properties.lefttile]
	local middle_tile = hajtech.map.tiles[self.gid]
	local right_tile = hajtech.map.tiles[self.gid + self.properties.righttile]

	for tx=0, self.width-1, tw do
		for ty=0, self.height-1, th do
			tile = middle_tile

			if self.width > tw then
				if tx == 0 then tile = left_tile end
				if tx >= self.width - tw then tile = right_tile end
			end

			-- todo: fix for tilesets with alternate size
			if tile.r == 0 then
				if tile.sx < 0 then tx = tx + tw end
				if tile.sy < 0 then ty = ty + th end
			else
				if tile.sx < 0 then ty = ty + tw end
				if tile.sy > 0 then tx = tx + th end
			end

			love.graphics.draw(
				tile.tileset.image,
				tile.quad,
				self.x + tx,
				self.y + ty,
				tile.r,
				tile.sx,
				tile.sy)
		end
	end
end

return platform
