
require("utils")

hajtech = {}
hajtech.rendering = require("rendering")
hajtech.tiledmap = require("tiledmap")
hajtech.viewport = require("viewport")
hajtech.input = require("input")
hajtech.collision = require("collision")
hajtech.animation = require("animation")
hajtech.minimap = require("minimap")
hajtech.titlemenu = require("titlemenu")
hajtech.gameendscreen = require("gameendscreen")
hajtech.persistent = require("persistent")
hajtech.transition = require("transition")
hajtech.charactermanager = require("charactermanager")
hajtech.cinematic = require("cinematic")
hajtech.fonts = {}
hajtech.fonts.segment = love.graphics.newImageFont("img/7seg-font.png", "0123456789:.-")
hajtech.fonts.sign = love.graphics.newImageFont("img/raven-font.png",
	"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./!?&: ")
hajtech.grayscale = love.graphics.newShader([[
	vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords)
	{
		vec4 texturecolor = Texel(texture, texture_coords);
		float grayscale = (texturecolor.x+texturecolor.y+texturecolor.z) / 3;
		return vec4(grayscale, grayscale, grayscale, texturecolor.w);
	}
]])

local map
local viewports = {}
local radar

hajtech.viewports = viewports

local mainbackground = love.graphics.newImage("img/border.png")
local control_panel = love.graphics.newImage("img/control_panel.png")

function hajtech.change_level(nextlevel, spawnprefix)
	spawnprefix = spawnprefix or hajtech.mapname or ""

	hajtech.mapname = nextlevel
	map = hajtech.tiledmap.loadMap(nextlevel)
	hajtech.map = map
	hajtech.collision.setMap(map)

	local spawndoor = map.namedentities[spawnprefix]
	local spawnpoints = {}
	for _, entity in ipairs(spawndoor and spawndoor.outgoing_links or {}) do
		if entity.type == "spawn" then
			table.insert(spawnpoints, entity)
		end
	end
	if #spawnpoints < 3 then
		spawnpoints = {
			map.namedentities["player1"],
			map.namedentities["player2"],
			map.namedentities["player3"],
			map.namedentities["player4"],
		}
	end

	assert(#spawnpoints >= 3, "Too few spawnpoints")

	hajtech.charactermanager.spawn_characters(map, spawnpoints)

	radar = hajtech.minimap.newMinimap(206, 206, map)

	if love.update then
		love.update(0)
	end
end

function love.load()
	hajtech.rendering.initialize(960, 540)
	love.graphics.setNewFont(16)
	map = hajtech.tiledmap.loadMap("stage_select")
	hajtech.map = map
	hajtech.collision.setMap(map)
	hajtech.input.loadBindings()

	local cm = hajtech.charactermanager
	cm.initialize()

	viewports[1] = hajtech.viewport.newViewport(430, 230, cm.characters[1])
	viewports[2] = hajtech.viewport.newViewport(430, 230, cm.characters[2])
	viewports[3] = hajtech.viewport.newViewport(430, 230, cm.characters[3])

	radar = hajtech.minimap.newMinimap(206, 206, map)

	hajtech.titlemenu.entermenu()
end

function love.draw()
	hajtech.rendering.drawbegin()

	love.graphics.draw(mainbackground, 0, 0)

	viewports[1]:draw(map, 36, 0)
	viewports[2]:draw(map, 494, 0)
	viewports[3]:draw(map, 36, 282)

	radar:draw()
	love.graphics.draw(radar.canvas, 701, 308)
	love.graphics.draw(control_panel, 494, 282)

	local oldfont = love.graphics.getFont()
	local r, g, b = love.graphics.getColor()
	do
		love.graphics.setColor(love.math.colorFromBytes(36, 34, 33))
		local level = hajtech.persistent.getCurrentLevelname()
		love.graphics.printf(level or "Hub", 518, 316, 148, 'left')

		love.graphics.setFont(hajtech.fonts.segment)
		love.graphics.setColor(love.math.colorFromBytes(133, 38, 38))

		local flasks = hajtech.persistent.getTotalFlasks()
		love.graphics.printf(flasks, 564, 381, 35, 'right')
		local time = hajtech.persistent.getCurrentTime() or "-:--:--"
		love.graphics.printf(time, 518, 446, 74, 'right')
		local record = hajtech.persistent.getCurrentRecord() or "-:--:--"
		love.graphics.printf(record, 518, 489, 74, 'right')
	end
	love.graphics.setFont(oldfont)
	love.graphics.setColor(r, g, b)

	hajtech.rendering.drawend()
end

function love.update(dt)
	map:update(dt)

	viewports[1]:follow()
	viewports[2]:follow()
	viewports[3]:follow()

	radar:update(dt)

	hajtech.persistent.update(dt)
end

function handleInput(playerid, action, prefix)
	assert(playerid >= 1 and playerid <= #viewports, "invalid playerid: " .. playerid)
	local player = hajtech.charactermanager.players[playerid]

	if prefix .. action == "press_swap" then
		-- Try to swap which viewport player controls
		for i=1,2 do
			local new_viewportid = mod(player.viewport + i, 3)
			local new_viewport = hajtech.viewports[new_viewportid]
			if not new_viewport.character.player then
				local old_viewport = hajtech.viewports[player.viewport]
				old_viewport.player = nil
				old_viewport.character.player = nil
				player.viewport = new_viewportid
				player.character = new_viewport.character
				new_viewport.player = playerid
				new_viewport.character.player = playerid
				return
			end
		end
		return
	end

	local character = player.character
	local handler = character[prefix .. action]
	if handler then
		handler(character)
	end
end

function love.keypressed(key, scancode)
	if key == 'escape' then
		hajtech.titlemenu.entermenu()
		return
	end

	if key == 'return' then
		if love.keyboard.isDown('lalt') or love.keyboard.isDown('ralt') then
			love.window.setFullscreen(not love.window.getFullscreen())
			return
		end
	end

	local playerid, action = hajtech.input.getkeymapping(key, scancode)
	if action then
		handleInput(playerid, action, "press_")
	end
end

function love.keyreleased(key, scancode)
	local playerid, action = hajtech.input.getkeymapping(key, scancode)
	if action then
		handleInput(playerid, action, "release_")
	end
end

function love.gamepadpressed(joystick, button)
	local playerid, action = hajtech.input.gamepadpressed(joystick, button)
	--if playerid then print(string.format("Player %i - %s", playerid, action or "")) end
	if action then
		handleInput(playerid, action, "press_")
	end

	if button == 'start' then
		hajtech.titlemenu.entermenu()
		return
	end
	-- if button == 'back' then
	-- 	hajtech.titlemenu.exitgame()
	-- 	return
	-- end
end

function love.gamepadreleased(joystick, button)
	local playerid, action = hajtech.input.gamepadpressed(joystick, button)
	--if playerid then print(string.format("Player %i - %s", playerid, action or "")) end
	if action then
		handleInput(playerid, action, "release_")
	end
end

function love.touchmoved(id, x, y, dx, dy, pressure)
	collision.moveX(characters[3], dx)
	collision.moveY(characters[3], dy)
end
