
local minimap = {}

local radarfade = love.graphics.newImage("img/radar_fade2.png")
radarfade:setFilter("nearest", "nearest")
local radaroverlay = love.graphics.newImage("img/radar.png")

function minimap.newMinimap(width, height, map)
	local self = {
		width = width,
		height = height,
		map = map,
		grid = {},
		fog = {},
		angle = 0,
	}

	self.canvas = love.graphics.newCanvas(self.width, self.height)
	self.canvas:setFilter("nearest", "nearest")

	self.draw = minimap.draw
	self.update = minimap.update
	self.discover = minimap.discover

	local gridmapping = {
		solid = {0, 128, 0},
		ledge = {20, 200, 20},
		ladder = {100, 128, 0},
	}

	for y = 1, map.height do
		self.grid[y] = {}
		self.fog[y] = {}
		for x = 1, map.width do
			self.grid[y][x] = false
			self.fog[y][x] = true
			for _, layer in ipairs(map.layers) do
				if layer.type == "tilelayer" and layer.properties.solid then
					local tile = layer.data[y][x]

					if tile then
						for property, value in pairs(tile.properties) do
							if gridmapping[property] then
								self.grid[y][x] = gridmapping[property]
								break
							end
						end
					end
				end
			end
		end
	end

	self.blips = {}
	for i = 1, 3 do
		local blip = {
			decay = 100,
			oldda = 0,
			da = 100,
		}
		blip.name = "character" .. i
		blip.entity = map.namedentities[blip.name]
		self.blips[#self.blips+1] = blip
	end
	self.blips[1].color = {love.math.colorFromBytes({149, 39, 54})}
	self.blips[2].color = {love.math.colorFromBytes({100, 125, 174})}
	self.blips[3].color = {love.math.colorFromBytes({196, 196, 196})}

	return self
end

function minimap:draw()
	local oldcanvas = love.graphics.getCanvas()
	love.graphics.setCanvas(self.canvas)
	love.graphics.clear({0, 0, 0})

	local cw, ch = self.canvas:getDimensions()
	local sx = self.width / self.map.width
	local sy = self.height / self.map.height

	for y = 1, self.map.height do
		for x = 1, self.map.width do
			if not self.fog[y][x] then
				local color = self.grid[y][x]
				if color then
					love.graphics.setColor(color)
					love.graphics.rectangle('fill', (x-1)*sx, (y-1)*sy, sx, sy)
				end
			end
		end
	end

	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(radarfade, cw/2, ch/2, self.angle, 4, 4, 100, 100)

	for _, blip in ipairs(self.blips) do
		local color = blip.color
		local x, y = math.floor(blip.x), math.floor(blip.y)
		color[4] = math.max(0, 255 - blip.decay * 50) / 255
		love.graphics.setColor(color)
		love.graphics.circle('fill', x, y, 5, 16)
		color[4] = math.max(0, 255 - blip.decay * 200) / 255
		love.graphics.setColor(color)
		love.graphics.circle('line', x, y, 5+blip.decay*10, 20)
	end

	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(radaroverlay, cw/2, ch/2, 0, 1, 1, 111, 111)

	local oldwidth = love.graphics.getLineWidth()
	local oldstyle = love.graphics.getLineStyle()
	love.graphics.setLineWidth(2)
	love.graphics.setLineStyle('rough')
	do
		love.graphics.setColor(love.math.colorFromBytes({0, 200, 0, 128}))
		local x1, y1 = cw / 2, ch / 2
		local x2 = x1 + math.cos(self.angle) * cw
		local y2 = y1 + math.sin(self.angle) * cw
		love.graphics.line(x1, y1, x2, y2)
	end
	love.graphics.setLineWidth(oldwidth)
	love.graphics.setLineStyle(oldstyle)

	love.graphics.setCanvas(oldcanvas)
	love.graphics.setColor(1, 1, 1)
end

function minimap:update(dt)
	local function normalizeangle(angle)
		while angle < 0 do angle = angle + 2*math.pi end
		while angle > 2*math.pi do angle = angle - 2*math.pi end
		return angle
	end

	self.angle = self.angle + 2*dt
	self.angle = normalizeangle(self.angle)

	local sx = self.width / (self.map.width * self.map.tilewidth)
	local sy = self.height / (self.map.height * self.map.tileheight)
	local cw, ch = self.canvas:getDimensions()

	for i = #self.blips, 1, -1 do
		local blip = self.blips[i]
		local entity = blip.entity

		local x, y = entity.x*sx, entity.y*sy

		local dx, dy = x - cw/2, y - ch/2
		local angle = math.atan2(dy, dx)

		blip.oldda = blip.da
		blip.da = normalizeangle(self.angle - angle)
		if blip.da < blip.oldda then
			blip.decay = 0
			blip.x, blip.y = x, y

			self:discover(entity.x, entity.y, 128)
		else
			blip.decay = blip.decay + dt
		end
	end
end

function minimap:discover(x, y, d)
	local tx = math.floor(x / self.map.tilewidth) + 1
	local ty = math.floor(y / self.map.tileheight) + 1
	local td = math.floor(d / self.map.tilewidth)

	for x = math.max(1, tx - td), math.min(self.map.width, tx + td) do
		for y = math.max(1, ty - td), math.min(self.map.height, ty + td) do
			self.fog[y][x] = false
		end
	end
end

return minimap
