
local tiledmap = {}

function tiledmap.loadMap(mapname)
	local map = require("maps." .. mapname)
	package.loaded["maps." .. mapname] = nil

	assert(map.orientation == "orthogonal", "no support for map orientation: " .. map.orientation)

	local function fixpath(tiledpath)
		local filepath = "maps/" .. tiledpath
		local subs
		repeat
			filepath, subs = string.gsub(filepath, "[^/]+/%.%./", "")
		until subs == 0
		return filepath
	end

	map.tiles = {}
	local gid
	for _, tileset in ipairs(map.tilesets) do
		if tileset.image then
			local tw = tileset.tilewidth
			local th = tileset.tileheight
			local iw = tileset.imagewidth
			local ih = tileset.imageheight
			local m = tileset.margin
			local s = tileset.spacing
			local w = (iw - m + s) / (tw + s)
			local h = (ih - m + s) / (th + s)

			local filepath = fixpath(tileset.image)
			tileset.image = love.graphics.newImage(filepath)
			tileset.image:setFilter("nearest", "nearest")

			gid = tileset.firstgid
			for y = 1, h do
				for x = 1, w do
					local qx = (x - 1) * (tw + s) + m
					local qy = (y - 1) * (th + s) + m

					local tile = {
						id         = gid - tileset.firstgid,
						gid        = gid,
						tileset    = tileset,
						quad       = love.graphics.newQuad(qx, qy, tw, th, iw, ih),
						sx         = 1,
						sy         = 1,
						r          = 0,
						properties = {},
					}

					for _, t in ipairs(tileset.tiles) do
						if t.id == tile.id then
							tile.properties = t.properties or tile.properties
							tile.animation = t.animation

							if tile.animation then
								tile.frame = 0 -- special value first time
								tile.time = 0
								tile.original_quad = tile.quad

								for _, frame in ipairs(tile.animation) do
									frame.tileid = frame.tileid + tileset.firstgid
									frame.duration = tonumber(frame.duration)
								end
							end
							break
						end
					end

					map.tiles[gid] = tile
					gid = gid + 1
				end
			end
		else
			-- No tileset.image => Image Collection
			gid = tileset.firstgid
			for _, tile in ipairs(tileset.tiles) do
				tile.properties = tile.properties or {}
				map.tiles[tile.id + gid] = tile
			end
		end
	end

	local function generate_flipped_gid(gid)
		local bit31 = 2^31
		local bit30 = 2^30
		local bit29 = 2^29
		local flipx = false
		local flipy = false
		local flipd = false
		local sourcegid = gid

		if sourcegid >= bit31 then
			sourcegid = sourcegid - bit31
			flipx = true
		end
		if sourcegid >= bit30 then
			sourcegid = sourcegid - bit30
			flipy = true
		end
		if sourcegid >= bit29 then
			sourcegid = sourcegid - bit29
			flipd = true
		end

		local sourcetile = map.tiles[sourcegid]
		local tile = {
			id         = sourcetile.id,
			gid        = gid,
			tileset    = sourcetile.tileset,
			quad       = sourcetile.quad,
			sx         = sourcetile.sx,
			sy         = sourcetile.sy,
			r          = sourcetile.r,
			properties = sourcetile.properties,

			animation  = sourcetile.animation,
			frame      = sourcetile.frame,
			time       = sourcetile.time,
			original_quad = sourcetile.original_quad,
		}

		if flipd then
			tile.r = math.rad(90)
			flipx, flipy = flipy, not flipx
		end
		if flipx then tile.sx = -1 end
		if flipy then tile.sy = -1 end

		map.tiles[gid] = tile
		return map.tiles[gid]
	end

	local function uint32_to_table(d)
		local t = {}
		local b = string.byte
		for i = 1, d:len(), 4 do
			local v = b(d, i) + b(d, i+1)*256 + b(d, i+2)*65536 + b(d, i+3)*16777216
			table.insert(t, v)
		end
		return t
	end

	map.namedentities = {}
	for _, layer in ipairs(map.layers) do
		if layer.type == "tilelayer" then
			if layer.encoding and layer.encoding == "base64" then
				if layer.compression then
					layer.data = love.data.decode("data", "base64", layer.data)
					layer.data = love.data.decompress("string", layer.compression, layer.data)
				else
					layer.data = love.data.decode("string", "base64", layer.data)
				end
				layer.data = uint32_to_table(layer.data)
			end

			local i = 1
			local data = {}

			for y = 1, layer.height do
				data[y] = {}
				for x = 1, layer.width do
					local gid = layer.data[i]
					if gid > 0 then
						data[y][x] = map.tiles[gid] or generate_flipped_gid(gid)
					end
					i = i + 1
				end
			end

			layer.data = data
		elseif layer.type == "objectgroup" then
			layer.entities = {}

			for _, object in ipairs(layer.objects) do
				object.layer = layer
				object.w, object.h = object.width, object.height
				if object.gid then
					if object.rotation == 0 then
						object.y = object.y - object.h
					elseif object.rotation == 90 then
						object.w, object.h = object.h, object.w
					elseif object.rotation == 180 then
						object.x = object.x - object.w
					elseif object.rotation == -90 then
						object.w, object.h = object.h, object.w
						object.x = object.x - object.w
						object.y = object.y - object.h
					end
					local tile = map.tiles[object.gid]
					for key, value in pairs(tile.properties) do
						object.properties[key] = object.properties[key] or value
					end
				end

				if not (#object.type > 0) then
					object.type = object.properties.type
				end
				if object.type and #object.type > 0 then
					local object_factory = require("entities/" .. object.type)
					object_factory.create(object)
				end

				if object.shape == "polyline" then
					local findEntities = hajtech.collision.entitiesAtPoint
					local startingpoint = object.polyline[1]
					local sources = findEntities(object.x + startingpoint.x,
						object.y + startingpoint.y, map)
					for i = 2, #object.polyline do
						local targetpoint = object.polyline[i]
						local targets = findEntities(object.x + targetpoint.x,
							object.y + targetpoint.y, map)
						for _, target in ipairs(targets) do
							if object.properties.follow then
								sources[1].followers = sources[1].followers or {}
								table.insert(sources[1].followers, target)
							else
								target.incoming_links = target.incoming_links or {}
								for _, source in ipairs(sources) do
									if source ~= target then
										source.outgoing_links = source.outgoing_links or {}
										table.insert(source.outgoing_links, target)
										if not object.properties.oneway then
											table.insert(target.incoming_links, source)
										end
									end
								end
							end
						end
					end
				else
					layer.entities[object] = object
					if object.name and #object.name > 0 then
						map.namedentities[object.name] = object
					end
				end
			end
		else
			error("no support for layer type: " .. layer.type)
		end
	end

	map.draw = tiledmap.draw
	map.update = tiledmap.update
	map.addEntity = tiledmap.addEntity
	map.removeEntity = tiledmap.removeEntity

	return map
end

function tiledmap.draw(map, drawrange)
	local tw = map.tilewidth
	local th = map.tileheight

	-- todo: maybe remove this and instead let viewport limit translation?
	drawrange.sx = math.max(drawrange.sx, 1)
	drawrange.sy = math.max(drawrange.sy, 1)
	drawrange.ex = math.min(drawrange.ex, map.width)
	drawrange.ey = math.min(drawrange.ey, map.height)

	for _, layer in ipairs(map.layers) do
		if layer.type == "tilelayer" then
			love.graphics.push("all")
			love.graphics.translate(layer.offsetx, layer.offsety)
			love.graphics.setColor(1, 1, 1, layer.opacity)
			for y = drawrange.sy, drawrange.ey do
				for x = drawrange.sx, drawrange.ex do
					local tile = layer.data[y][x]
					if tile then
						local tx = (x - 1) * tw
						local ty = (y - 1) * th

						-- todo: fix for tilesets with alternate size
						if tile.r == 0 then
							if tile.sx < 0 then tx = tx + tw end
							if tile.sy < 0 then ty = ty + th end
						else
							if tile.sx < 0 then ty = ty + tw end
							if tile.sy > 0 then tx = tx + th end
						end

						love.graphics.draw(
							tile.tileset.image,
							tile.quad,
							tx,
							ty,
							tile.r,
							tile.sx,
							tile.sy)
					end
				end
			end
			love.graphics.pop()
		elseif layer.type == "objectgroup" then
			for _, entity in pairs(layer.entities) do
				if entity.visible then
					if entity.draw then
						entity:draw()
					elseif entity.image then
						love.graphics.draw(entity.image,
							math.floor(entity.x),
							math.floor(entity.y))
					elseif entity.gid then
						local tile = map.tiles[entity.gid]

						if tile then
							local tx = entity.x
							local ty = entity.y

							-- todo: fix for tilesets with alternate size
							if tile.r == 0 then
								if tile.sx < 0 then tx = tx + tw end
								if tile.sy < 0 then ty = ty + th end
							else
								if tile.sx < 0 then ty = ty + tw end
								if tile.sy > 0 then tx = tx + th end
							end

							love.graphics.draw(
								tile.tileset.image,
								tile.quad,
								tx,
								ty,
								tile.r,
								tile.sx,
								tile.sy)
						end
					end
				end
			end
		end
	end
end

function tiledmap.update(map, dt)
	for gid, tile in pairs(map.tiles) do
		if tile.animation then
			local update = false
			tile.time = tile.time + dt * 1000

			-- first frame initialization
			if tile.frame == 0 then
				tile.frame = 1
				update = true
			end

			while tile.time > tile.animation[tile.frame].duration do
				update = true
				tile.time = tile.time - tile.animation[tile.frame].duration
				tile.frame = tile.frame + 1
				if tile.frame > #tile.animation then tile.frame = 1 end
			end

			if update then
				local newgid = tile.animation[tile.frame].tileid
				local newtile = map.tiles[newgid]
				tile.quad = newtile.original_quad or newtile.quad
			end
		end
	end

	for _, layer in ipairs(map.layers) do
		if layer.type == "objectgroup" then
			for _, entity in pairs(layer.entities) do
				if entity.update then
					entity:update(dt)
				end
			end
		end
	end
end

function tiledmap.addEntity(map, entity, layer)
	layer = layer or entity.layer
	entity.properties = entity.properties or {}
	layer.entities[entity] = entity
	if entity.name and #entity.name > 0 then
		map.namedentities[entity.name] = entity
	end
end

function tiledmap.removeEntity(map, entity, layer)
	layer = layer or entity.layer
	layer.entities[entity] = nil
	if entity.name and #entity.name > 0 then
		if map.namedentities[entity.name] == entity then
			map.namedentities[entity.name] = nil
		end
	end
end

function tiledmap.findTile(map, x, y)
	tx = math.floor(x / map.tilewidth) + 1
	ty = math.floor(y / map.tileheight) + 1

	for i = #map.layers, 1, -1 do
		local tile = map.layers[i].data[ty][tx]
		if tile then
			return tile
		end
	end
end

return tiledmap
